=== modified file 'NEWS'
--- NEWS	2007-12-22 02:01:03 +0000
+++ NEWS	2008-01-11 03:01:54 +0000
@@ -1,3 +1,6 @@
+Jan 10
+* Support branch6 formats (dirstate-tags, pack*) in rspush
+
 Dec 21
 * "branches" and "multi-pull" updated to new find_* APIs.
 * new "trees" command.

=== modified file 'bzrtools.py'
--- bzrtools.py	2007-12-22 17:55:17 +0000
+++ bzrtools.py	2008-01-11 03:01:54 +0000
@@ -24,7 +24,7 @@
 import sys
 
 import bzrlib
-from bzrlib import trace, urlutils
+from bzrlib import revision as _mod_revision, trace, urlutils
 import bzrlib.errors
 from bzrlib.errors import (
     BzrCommandError,
@@ -112,22 +112,6 @@
 
 def rsync(source, target, ssh=False, excludes=(), silent=False,
           rsync_name="rsync"):
-    """
-    >>> new_dir = tempfile.mkdtemp()
-    >>> old_dir = os.getcwd()
-    >>> os.chdir(new_dir)
-    >>> rsync("a", "b", silent=True)
-    Traceback (most recent call last):
-    RsyncNoFile: No such file...
-    >>> rsync(new_dir + "/a", new_dir + "/b", excludes=("*.py",), silent=True)
-    Traceback (most recent call last):
-    RsyncNoFile: No such file...
-    >>> rsync(new_dir + "/a", new_dir + "/b", excludes=("*.py",), silent=True, rsync_name="rsyncc")
-    Traceback (most recent call last):
-    NoRsync: rsyncc not found.
-    >>> os.chdir(old_dir)
-    >>> os.rmdir(new_dir)
-    """
     cmd = [rsync_name, "-av", "--delete"]
     if ssh:
         cmd.extend(('-e', 'ssh'))
@@ -197,6 +181,15 @@
     return [l.rstrip('\r\n') for l in
             codecs.open(fname, 'rb', 'utf-8').readlines()]
 
+
+def read_revision_info(path):
+    """Parse a last_revision file to determine revision_info"""
+    line = open(path, 'rb').readlines()[0].strip('\n')
+    revno, revision_id = line.split(' ', 1)
+    revno = int(revno)
+    return revno, revision_id
+
+
 class RsyncNoFile(Exception):
     def __init__(self, path):
         Exception.__init__(self, "No such file %s" % path)
@@ -234,15 +227,38 @@
     return history
 
 
+def get_revision_info(location, _rsync):
+    """Get the revsision_info for an rsync-able branch"""
+    tempdir = tempfile.mkdtemp('push')
+    my_rsync = _rsync
+    if my_rsync is None:
+        my_rsync = rsync
+    try:
+        info_fname = os.path.join(tempdir, 'last-revision')
+        cmd = rsync(location+'.bzr/branch/last-revision', info_fname,
+                    silent=True)
+        return read_revision_info(info_fname)
+    finally:
+        shutil.rmtree(tempdir)
+
+
 def history_subset(location, branch, _rsync=None):
-    remote_history = get_revision_history(location, _rsync)
     local_history = branch.revision_history()
-    if len(remote_history) > len(local_history):
-        return False
-    for local, remote in zip(remote_history, local_history):
-        if local != remote:
+    try:
+        remote_history = get_revision_history(location, _rsync)
+    except RsyncNoFile:
+        revno, revision_id = get_revision_info(location, _rsync)
+        if revision_id == _mod_revision.NULL_REVISION:
+            return True
+        return bool(revision_id.decode('utf-8') in local_history)
+    else:
+        if len(remote_history) > len(local_history):
             return False
-    return True
+        for local, remote in zip(remote_history, local_history):
+            if local != remote:
+                return False
+        return True
+
 
 def empty_or_absent(location):
     try:
@@ -316,7 +332,7 @@
                     ' "%s" is of the form "machine:/path".' % push_location)
         trace.note("Pushing to %s", push_location)
         my_rsync(tree.basedir+'/', push_location, ssh=usessh,
-              excludes=final_exclusions)
+                 excludes=final_exclusions)
 
         set_push_data(tree, push_location)
     finally:

=== modified file 'tests/test_rspush.py'
--- tests/test_rspush.py	2007-11-29 15:44:09 +0000
+++ tests/test_rspush.py	2008-01-11 03:01:54 +0000
@@ -15,10 +15,19 @@
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+import os
+
 from bzrlib.bzrdir import BzrDir
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
 
-from bzrlib.plugins.bzrtools.bzrtools import rspush, NotStandalone
+from bzrlib.plugins.bzrtools.bzrtools import (
+    history_subset,
+    NoRsync,
+    NotStandalone,
+    rspush,
+    rsync,
+    RsyncNoFile,
+)
 
 
 class MockRSync(object):
@@ -35,6 +44,18 @@
 
 class TestRSPush(TestCaseWithTransport):
 
+    def test_rsync(self):
+        cwd = os.getcwd()
+        try:
+            self.assertRaises(RsyncNoFile, rsync, "a", "b", silent=True)
+        except NoRsync:
+            raise TestSkipped('rsync not installed')
+        self.assertRaises(RsyncNoFile, rsync, cwd + "/a", cwd + "/b",
+                          excludes=("*.py",), silent=True)
+        self.assertRaises(NoRsync, rsync, cwd + "/a", cwd + "/b",
+                          excludes=("*.py",), silent=True,
+                          rsync_name="rsyncc")
+
     def test_rspush_locking(self):
         tree = self.make_branch_and_tree('tree', format='dirstate-tags')
         tree.commit('some changes')
@@ -72,3 +93,36 @@
         mock_rsync = MockRSync()
         self.assertRaises(NotStandalone, rspush, tree, 'example.org:foo',
                           _rsync=mock_rsync)
+
+    def do_history_subset(self, format):
+        tree_a = self.make_branch_and_tree('tree_a', format=format)
+        tree_a.lock_write()
+        self.addCleanup(tree_a.unlock)
+        tree_a.commit('foo')
+        tree_b = tree_a.bzrdir.sprout('tree_b').open_workingtree()
+        tree_a.commit('bar')
+        try:
+            self.assertTrue(history_subset('tree_b/', tree_a.branch))
+        except NoRsync:
+            raise TestSkipped('rsync not installed')
+        self.assertFalse(history_subset('tree_a/', tree_b.branch))
+
+    def do_history_subset_no_commits(self, format):
+        tree_a = self.make_branch_and_tree('tree_a', format=format)
+        tree_b = self.make_branch_and_tree('tree_b', format=format)
+        try:
+            self.assertTrue(history_subset('tree_b/', tree_a.branch))
+        except NoRsync:
+            raise TestSkipped('rsync not installed')
+
+    def test_history_subset_dirstate(self):
+        self.do_history_subset('dirstate')
+
+    def test_history_subset_no_commits_dirstate(self):
+        self.do_history_subset_no_commits('dirstate')
+
+    def test_history_subset_pack(self):
+        self.do_history_subset('pack-0.92')
+
+    def test_history_subset_no_commits_pack(self):
+        self.do_history_subset_no_commits('pack-0.92')

